package ApacheKit;

use ApacheKit::Base;
use ApacheKit::Find;
use ApacheKit::ErrorMessage;
use ApacheKit::Media;
use ApacheKit::Output;

# Imap
use ApacheKit::Imap;

# Apache and it's additional modules
use ApacheKit::Apache;
use ApacheKit::Php3;
use ApacheKit::Php4;
use ApacheKit::ModPerl;
use ApacheKit::Openssl;

@ISA = qw(ApacheKit::Base);

use strict;

sub _init {
   my $self = shift;

   $self->{ 'find' }       = ApacheKit::Find->new();
   $self->{ 'error' }      = ApacheKit::ErrorMessage->new();
   $self->{ 'output' }     = ApacheKit::Output->new();

   $self->{ 'apache' }     = ApacheKit::Apache->new();
   $self->{ 'php3' }       = ApacheKit::Php3->new();
   $self->{ 'php4' }       = ApacheKit::Php4->new();
   $self->{ 'imap' }       = ApacheKit::Imap->new();
   $self->{ 'mod_perl' }   = ApacheKit::ModPerl->new();
   $self->{ 'openssl' }    = ApacheKit::Openssl->new();

   $self->{ 'version' }             = '7.8';

}

sub run {
   my $self = shift;

   $self->{'output'}->start( 'Apache build kit version' );
   $self->{'output'}->stop( $self->{ 'version' } );

   # Find the required commands to run
   $self->find_required_commands();

   # Autodetection of libraries and features for php3 and other modules
   # is mysql available
   $self->is_mysql_available();
   # is gd available
   $self->is_gd_available();
   # is postgres available
   $self->is_postgres_available();
   # is oracle available
   $self->is_oracle_available();
   # is ldap available
   $self->is_ldap_available();
   # is aspell available
   $self->is_aspell_available();

   # Imap build is needed ...
   if ( $self->{ 'build_imap' } == 1 ) {
      $self->{ 'imap' }->copy_defaults( $self );
      $self->{ 'imap' }->build();
      # check for imap availability at end of build
   }

   # is imap available
   $self->is_imap_available();

   # Open ssl build
   if ( $self->{ 'build_openssl' } == 1 || $self->{ 'build_modssl' } == 1 ) {
      $self->{'openssl'}->copy_defaults( $self );
      $self->{'openssl'}->build();
   }

   # Build apache
   $self->{ 'apache' }->copy_defaults( $self );
   $self->{ 'apache' }->build( $self->{'openssl'}->{'openssl_src'} );

   if ( $self->{ 'build_php3' } == 1 ) {
      $self->{ 'php3' }->copy_defaults( $self );
      $self->{ 'php3' }->build();
      $self->{ 'apache' }->activate_addtype(
         'application/x-httpd-php3', '.php3'
      );
      $self->{ 'apache' }->add_dir_index_file( 'index.php3' );
   }

   if ( $self->{ 'build_php4' } == 1 ) {
      $self->{ 'php4' }->copy_defaults( $self );
      $self->{ 'php4' }->build();
      $self->{ 'apache' }->activate_addtype(
         'application/x-httpd-php', '.php4'
      );
      $self->{ 'apache' }->activate_addtype(
         'application/x-httpd-php', '.php'
      );
      $self->{ 'apache' }->add_dir_index_file( 'index.php' );
   }

   if ( $self->{ 'build_mod_perl' } == 1 ) {
      $self->{ 'mod_perl' }->copy_defaults( $self );
      $self->{ 'mod_perl' }->build();
   }

}

sub find_required_commands {
   my $self = shift;

   $self->{'output'}->start( 'Finding required commands' );

   $self->{ 'cp_command' }       = $self->{ 'find' }->command( 'cp' );
   $self->{ 'mkdir_command' }    = $self->{ 'find' }->command( 'mkdir' );
   $self->{ 'gunzip_command' }   = $self->{ 'find' }->command( 'gunzip' );
   $self->{ 'tar_command' }      = $self->{ 'find' }->command( 'tar' );
   $self->{ 'perl_command' }     = $self->{ 'find' }->command( 'perl' );

   if ( $self->{ 'tar_command' } eq 'NOT FOUND' ) {
      $self->{ 'tar_command' }      = $self->{ 'find' }->command( 'gtar' );
   }

   $self->{ 'error' }->command_not_found( 'cp',     $self->{ 'cp_command' } );
   $self->{ 'error' }->command_not_found( 'mkdir',  $self->{ 'mkdir_command' } );
   $self->{ 'error' }->command_not_found( 'gunzip', $self->{ 'gunzip_command' } );
   $self->{ 'error' }->command_not_found( 'tar',    $self->{ 'tar_command' } );
   $self->{ 'error' }->command_not_found( 'perl',   $self->{ 'perl_command' } );

   $self->{'output'}->stop( 'Done' );

}

sub feature_available {
   my $self          = shift;
   my $feature_name  = shift;
   my $feature       = shift;

   $self->{'output'}->start( $feature_name );
   if ( $self->{ $feature } == 1 ) {
      $self->{'output'}->stop( 'Yes' );
   } else {
      $self->{'output'}->stop( 'No' );
   }

}

sub is_aspell_available {
   my $self = shift;
# Change this ??: 
   my $aspell_c = $self->{'find'}->header( 'aspell-c.h' );

   if ( $aspell_c ne 'NOT FOUND' ) {
      $self->{'aspell_available'} = 1;
   }

   $self->feature_available(
      'Aspell available',
      'aspell_available'
   );

}

sub is_ldap_available {
   my $self = shift;

   $self->{ 'ldap_available' } = 0;

   my $libldap_la = $self->{ 'find' }->library( 'libldap.la' );

   if ( $libldap_la ne 'NOT FOUND' ) {
      $self->{ 'ldap_available' } = 1;
   }

   $self->feature_available(
      'LDAP available',
      'ldap_available'
   );

}

sub is_imap_available {
   my $self = shift;

   my $mail_h     = $self->{ 'find' }->header( 'mail.h' );
   my $rfc822_h   = $self->{ 'find' }->header( 'rfc822.h' );
   my $linkage_h  = $self->{ 'find' }->header( 'linkage.h' );
   my $c_client_a = $self->{ 'find' }->library( 'c-client.a' );

   #print "mail.h     : $mail_h\n";
   #print "rfc822.h   : $rfc822_h\n";
   #print "linkage.h  : $linkage_h\n";
   #print "c-client.a : $c_client_a\n";

   if ( $mail_h eq 'NOT FOUND' || $rfc822_h eq 'NOT FOUND' || $linkage_h eq 'NOT FOUND' ) {
      $self->{ 'imap_available' } = 0;
   } else {
      $self->{ 'imap_available' } = 1;
   }

   $self->feature_available(
      'Imap available',
      'imap_available'
   );

}

sub is_mysql_available {
   my $self = shift;

   my $mysql_h = $self->{ 'find' }->header( 'mysql/mysql.h' );

   if ( $mysql_h eq 'NOT FOUND' ) {
      $self->{ 'mysql_available' } = 0;
   } else {
      $self->{ 'mysql_available' } = 1;
   }

   $self->feature_available(
      'Mysql available',
      'mysql_available'
   );

}

sub is_gd_available {
   my $self = shift;

   my $gd_h = $self->{ 'find' }->header( 'gd.h' );

   if ( $gd_h eq 'NOT FOUND' ) {
      $self->{ 'gd_available' } = 0;
   } else {
      $self->{ 'gd_available' } = 1;
   }

   $self->feature_available(
      'Gd available',
      'gd_available'
   );

}

sub is_postgres_available {
   my $self = shift;

   my $postgres1_h = $self->{ 'find' }->header( 'postgres.h' );
   my $postgres2_h = $self->{ 'find' }->header( 'pgsql/postgres.h' );
   if ( $postgres1_h ne 'NOT FOUND' || $postgres2_h ne 'NOT FOUND' ) {
      $self->{ 'postgres_available' } = 1;
   } else {
      $self->{ 'postgres_available' } = 0;
   }

   $self->feature_available(
      'Postgres available',
      'postgres_available'
   );

}

sub is_oracle_available {
   my $self = shift;

   if ( $self->{ 'oracle_home' } eq '' ) {
      $self->{ 'oracle_home' } = $ENV{ 'ORACLE_HOME' };
   }

   if ( -d $self->{ 'oracle_home' } || -l $self->{ 'oracle_home' } ) {
      $self->{ 'oracle_available' } = 1;
   }

   $self->feature_available(
      'Oracle available',
      'oracle_available'
   );

}

sub display_commandline_options {
   my $self = shift;
   my $error = shift;

   print <<"COMM_OPTS";

$0 [os-platform] [options]

Os platform:

--redhat-7
--redhat-6
--redhat-5
--solaris-gcc

Apache:

--apache-dir=directory

Changes the directory that apache is placed in /usr/local/apache

Module compilation:

--build-openssl      Build open ssl libraries
--build-imap         Uw imap4 client libraries for imap support
--build-php3         Php3
--build-php4         Php4
--build-mod-perl     Mod perl
--build-mod-ssl      Mod SSL (Requires build of openssl)

Openssl:

--openssl-dir=directory

Changes the directory that openssl is installed to

COMM_OPTS

   if ( $error ne '' ) {
      print "\n*** " . $error . "\n";
   }

}

1;
